import errno
import json
import os
from pathlib import Path

import six
from termcolor import colored

STAC_SERVERS = {
    "pc": "Planetary Computer",
    "radiant": "Radiant MLHub",
}
CREDENTIAL_PATH = "~/.config/easystac/"


def get_credentials_path(stac_server: str = "radiant"):
    """Returns the path to the credentials file"""
    path_name = "%s/credentials_%s.json" % (CREDENTIAL_PATH, stac_server)
    return Path(path_name).expanduser()


def write_token(refresh_token: str, stac_server: str = "radiant"):
    """Attempts to write the passed token to the given user directory"""
    credentials_path = get_credentials_path(stac_server=stac_server)
    dirname = credentials_path.parent
    try:
        dirname.mkdir()
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise Exception("Error creating directory %s: %s" % (dirname, e))

    file_content = json.dumps({"token": refresh_token})
    # Remove file because os.open will not change permissions of existing files
    if credentials_path.exists():
        credentials_path.unlink()
    with os.fdopen(
        os.open(credentials_path, os.O_WRONLY | os.O_CREAT, 0o600), "w"
    ) as f:
        f.write(file_content)
    return True


def get_authorization_url(stac_server="pc"):
    urls = {
        "pc": "http://planetarycomputer.microsoft.com/compute/hub/token",
        "radiant": "https://mlhub.earth/profile",
    }
    return urls[stac_server]


def obtain_and_write_token(token=None, stac_server: str = "radiant"):
    """Obtains and writes credentials token based on an authorization code."""
    if token is None:
        display_auth_instructions(stac_server)
        token = input("Enter token: ")

    if isinstance(token, six.string_types):
        write_token(token, stac_server)
        print("\nSuccessfully saved authorization token.")
    else:
        raise Exception("Invalid token type: %s" % type(token))


# Display instructions for obtaining an authorization token ---------
def display_auth_instructions(stac_server="radiant"):
    ss_name = colored(STAC_SERVERS[stac_server], attrs=["bold"])
    if stac_server == "radiant":
        main_message = "To authorize access needed by %s, %s" % (
            ss_name,
            display_auth_instructions_radiant(get_authorization_url(stac_server)),
        )
        print(main_message)
    elif stac_server == "pc":
        main_message = "To authorize access needed by %s, %s" % (
            ss_name,
            display_auth_instructions_planetary(get_authorization_url(stac_server)),
        )
        print(main_message)
    return False


def display_auth_instructions_planetary(url):
    return (
        "open the following URL: %s in your web browser. "
        % colored(url, attrs=["bold"])
        + "An API key is required to have a more favorable rate limiting in Planetary Computer "
        + ", so please copy and paste it below."
    )


def display_auth_instructions_radiant(url):
    return (
        "open the following URL: %s in your web browser. "
        % colored(url, attrs=["bold"])
        + "An API key is required to access the Radiant MLHub API "
        + ", so please copy and paste it below."
    )



import json
import os
import warnings
from pathlib import Path

import planetary_computer as pc
import stackstac

from ..base import BaseImageCollection

warnings.simplefilter("always", UserWarning)


class ImageCollection(BaseImageCollection):
    """ImageCollection object for Planetary Computer.

    This object mimics the Earth Engine filtering methods for ee.ImageCollection class and
    makes them available for the Planetary Computer STAC. Items signing is automatically
    done.

    Parameters
    ----------
    collection: str
        Collection name.

    Examples
    --------
    >>> import easystac.planetary as pc
    >>> from geojson import Point
    >>> pc.Authenticate()
    >>> pc.Initialize()
    >>> geom = Point([-76.1,4.3])
    >>> S2 = (pc.ImageCollection("sentinel-2-l2a")
            .filterBounds(geom)
            .filterDate("2020-01-01","2021-01-01")
            .getInfo(resolution = 10))
    """

    def getInfo(self, **kwargs):
        """Returns all the information from the STAC search.

        Parameters
        ----------
        **kwargs
            Additional arguments passed to :code:`stackstac.stack()`. Some of them are
            :code:`epsg`, :code:`resolution`, and :code:`bbox`.

        Returns
        -------
        xarray.DataArray
            Chunked DataArray with Dask.

        Examples
        --------
        >>> import easystac.planetary as pc
        >>> from geojson import Point
        >>> pc.Authenticate()
        >>> pc.Initialize()
        >>> geom = Point([-76.1,4.3])
        >>> S2 = (pc.ImageCollection("sentinel-2-l2a")
                .filterBounds(geom)
                .filterDate("2020-01-01","2021-01-01")
                .getInfo(resolution = 10))
        """
        parameters = None

        if "PC_SDK_SUBSCRIPTION_KEY" in os.environ:
            parameters = {"subscription-key": os.environ["PC_SDK_SUBSCRIPTION_KEY"]}

        search = self._search(
            url="https://planetarycomputer.microsoft.com/api/stac/v1",
            parameters=parameters,
        )

        items = [pc.sign(item).to_dict() for item in search.get_items()]

        image_collection = stackstac.stack(items, **kwargs)

        return image_collection



import json
import os

import stackstac

from ..base import BaseImageCollection


class ImageCollection(BaseImageCollection):
    """ImageCollection object for Radiant Earth ML Hub.

    This object mimics the Earth Engine filtering methods for ee.ImageCollection class and
    makes them available for the Radiant Earth ML Hub STAC.

    Parameters
    ----------
    collection: str
        Collection name.

    Examples
    --------
    >>> import easystac.radiant as rd
    >>> rd.Authenticate()
    >>> rd.Initialize()
    >>> S1floods = (rd.ImageCollection("sen12floods_s1_source")
            .filterDate("2019-01-01","2019-01-05")
            .getInfo(epsg = 4326,resolution = 0.0001))
    """

    def getInfo(self, **kwargs):
        """Returns all the information from the STAC search.

        Parameters
        ----------
        **kwargs
            Additional arguments passed to :code:`stackstac.stack()`. Some of them are
            :code:`epsg`, :code:`resolution`, and :code:`bbox`.

        Returns
        -------
        xarray.DataArray
            Chunked DataArray with Dask.

        Examples
        --------
        >>> import easystac.radiant as rd
        >>> rd.Authenticate()
        >>> rd.Initialize()
        >>> S1floods = (rd.ImageCollection("sen12floods_s1_source")
                .filterDate("2019-01-01","2019-01-05")
                .getInfo(epsg = 4326,resolution = 0.0001))
        """
        search = self._search(
            url="https://api.radiant.earth/mlhub/v1/",
            parameters={"key": os.environ["MLHUB_API_KEY"]},
        )

        items = [item.to_dict() for item in search.get_items()]

        image_collection = stackstac.stack(items, **kwargs)

        return image_collection



import stackstac
from pystac_client import Client


class BaseImageCollection:
    """BaseImageCollection object.

    Base object for ImageCollection objects. This object mimics the Earth Engine
    filtering methods for ee.ImageCollection class and makes them available for other
    STAC.

    Parameters
    ----------
    collection: str
        Collection name.

    Examples
    --------
    >>> import easystac.planetary as pc
    >>> from geojson import Point
    >>> pc.Authenticate()
    >>> pc.Initialize()
    >>> geom = Point([-76.1,4.3])
    >>> S2 = (pc.ImageCollection("sentinel-2-l2a")
            .filterBounds(geom)
            .filterDate("2020-01-01","2021-01-01")
            .getInfo(resolution = 10))
    """

    def __init__(self, collection):
        """Initializes the BaseImageCollection object."""

        self.collection = collection
        """Collection ID."""

        self.datetime = None
        """Datetime string used to search items."""

        self.geometry = None
        """GeoJSON geometry used to search items."""

    def filterDate(self, initialDate, finalDate):
        """Initializes the initial and final date for datetime filtering.

        Parameters
        ----------
        initialDate: str
            Initial date in format %Y-%m-%d.
        finalDate: str
            Final date in format %Y-%m-%d.

        Returns
        -------
        BaseImageCollection

        Examples
        --------
        >>> import easystac.planetary as pc
        >>> pc.Authenticate()
        >>> pc.Initialize()
        >>> S2 = (pc.ImageCollection("sentinel-2-l2a")
                .filterDate("2020-01-01","2021-01-01"))
        """
        self.datetime = f"{initialDate}/{finalDate}"

        return self

    def filterBounds(self, geometry):
        """Initializes the geometry for bounds filtering.

        Parameters
        ----------
        geometry: dict
            GeoJSON object of dictionary-like object representing a GeoJSON.

        Returns
        -------
        BaseImageCollection

        Examples
        --------
        >>> import easystac.planetary as pc
        >>> from geojson import Point
        >>> pc.Authenticate()
        >>> pc.Initialize()
        >>> geom = Point([-76.1,4.3])
        >>> S2 = (pc.ImageCollection("sentinel-2-l2a")
                .filterBounds(geom))
        """
        self.geometry = geometry

        return self

    def _search(self, url, parameters=None):
        """Computes the searching process through the STAC catalog.

        Parameters
        ----------
        url: str
            STAC Catalog url.
        parameters: dict
            Dictionary of query parameters.

        Returns
        -------
        ItemSearch
        """
        catalog = Client.open(url, parameters=parameters)

        search = catalog.search(
            intersects=self.geometry,
            datetime=self.datetime,
            collections=[self.collection],
        )

        return search


class ImageCollection(BaseImageCollection):
    """ImageCollection object for any kind of STAC.

    This object mimics the Earth Engine filtering methods for ee.ImageCollection class and
    makes them available for any kind of STAC.

    Parameters
    ----------
    collection: str
        Collection name.

    Examples
    --------
    >>> import easystac as es
    >>> from geojson import Polygon
    >>> geom = Polygon([
    >>>         [
    >>>             [-122.1553, 38.7578],
    >>>             [-121.8321, 39.7444],
    >>>             [-123.0002, 39.7503],
    >>>             [-123.0002, 38.7609],
    >>>             [-122.1553, 38.7578]
    >>>         ]
    >>>     ]
    >>> )
    >>> HLSS30 = (es.ImageCollection("HLSS30.v2.0")
    >>>     .fromSTAC("https://cmr.earthdata.nasa.gov/stac/LPCLOUD/")
    >>>     .filterBounds(geom)
    >>>     .filterDate("2021-01-01","2022-01-01")
    >>>     .getInfo(epsg = 4326,resolution = 0.0001,assets = ["B02","B03","B04"]))
    """

    def fromSTAC(self, url):
        """Initializes the STAC for querying.

        Parameters
        ----------
        url: str
            STAC Catalog url.

        Returns
        -------
        ImageCollection

        Examples
        --------
        >>> import easystac as es
        >>> HLSS30 = (es.ImageCollection("HLSS30.v2.0")
        >>>     .fromSTAC("https://cmr.earthdata.nasa.gov/stac/LPCLOUD/")
        """
        self.stac = url
        """STAC url used to search items."""

        return self

    def getInfo(self, **kwargs):
        """Returns all the information from the STAC search.

        Parameters
        ----------
        **kwargs
            Additional arguments passed to :code:`stackstac.stack()`. Some of them are
            :code:`epsg`, :code:`resolution`, and :code:`bbox`.

        Returns
        -------
        xarray.DataArray
            Chunked DataArray with Dask.

        Examples
        --------
        >>> import easystac as es
        >>> from geojson import Polygon
        >>> geom = Polygon([
        >>>         [
        >>>             [-122.1553, 38.7578],
        >>>             [-121.8321, 39.7444],
        >>>             [-123.0002, 39.7503],
        >>>             [-123.0002, 38.7609],
        >>>             [-122.1553, 38.7578]
        >>>         ]
        >>>     ]
        >>> )
        >>> HLSS30 = (es.ImageCollection("HLSS30.v2.0")
        >>>     .fromSTAC("https://cmr.earthdata.nasa.gov/stac/LPCLOUD/")
        >>>     .filterBounds(geom)
        >>>     .filterDate("2021-01-01","2022-01-01")
        >>>     .getInfo(epsg = 4326,resolution = 0.0001,assets = ["B02","B03","B04"]))
        """
        search = self._search(url=self.stac)

        items = [item.to_dict() for item in search.get_items()]

        image_collection = stackstac.stack(items, **kwargs)

        return image_collection



"""easystac - Planetary Computer authentication"""
import json
import os
import warnings
from pathlib import Path

from ..logging_utils import obtain_and_write_token
from .image_collection import ImageCollection

warnings.simplefilter("always", UserWarning)

CREDENTIAL_FILE = "~/.config/easystac/credentials_pc.json"


def Authenticate(token=None):
    """Generates an authentication prompt to Planetary Computer.

    Examples
    --------
    >>> import easystac.planetary as pc
    >>> pc.Authenticate()
    """
    if token is None:
        obtain_and_write_token(stac_server="pc")
    else:
        obtain_and_write_token(token, stac_server="pc")


def Initialize():
    """Initializes the authentication process for Planetary Computer.

    Examples
    --------
    >>> import easystac.planetary as pc
    >>> pc.Initialize()
    """
    credentials_path = Path(CREDENTIAL_FILE).expanduser()
    if credentials_path.is_file():
        credential = json.load(open(credentials_path))
        os.environ["PC_SDK_SUBSCRIPTION_KEY"] = credential["token"]
    else:
        warnings.warn(
            "PC_SDK_SUBSCRIPTION_KEY is not set.\n"
            + "If you are a registered user,"
            + " it is recommended to set your token for a more favorable rate limiting.\n"
            + "More info in https://planetarycomputer.microsoft.com/docs/concepts/sas/",
            UserWarning,
        )



import io
import os
import re

from setuptools import find_packages, setup


def read(filename):
    filename = os.path.join(os.path.dirname(__file__), filename)
    text_type = type(u"")
    with io.open(filename, mode="r", encoding="utf-8") as fd:
        return re.sub(text_type(r":[a-z]+:`~?(.*?)`"), text_type(r"``\1``"), fd.read())


setup(
    name="easystac",
    version="0.0.1",
    url="https://github.com/cloudsen12/easystac",
    license="MIT",
    author="David Montero Loaiza and Cesar Aybar Camacho",
    author_email="dml.mont@gmail.com",
    description="A Python package for simple STAC queries",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=("tests",)),
    install_requires=[
        "planetary_computer",
        "pystac_client",
        "radiant_mlhub",
        "six",
        "stackstac",
        "termcolor",
    ],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)



"""easystac - RadiantML authentication"""
import json
import os
from pathlib import Path

import stackstac

from ..logging_utils import obtain_and_write_token
from .image_collection import ImageCollection

CREDENTIAL_FILE = "~/.config/easystac/credentials_radiant.json"


def Authenticate(token=None):
    """Generates an authentication prompt to Radiant ML Hub.

    Examples
    --------
    >>> import easystac.radiant as rd
    >>> rd.Authenticate()
    """
    if token is None:
        obtain_and_write_token(stac_server="radiant")
    else:
        obtain_and_write_token(token, stac_server="radiant")


def Initialize():
    """Initializes the authentication process for Radiant ML Hub.

    Examples
    --------
    >>> import easystac.radiant as rd
    >>> rd.Initialize()
    """
    credentials_path = Path(CREDENTIAL_FILE).expanduser()
    if credentials_path.is_file():
        credential = json.load(open(credentials_path))
        os.environ["MLHUB_API_KEY"] = credential["token"]
    else:
        raise Exception(
            "Please authorize access to your radiantHub account by "
            "running:: \n\n>>> import easystac.radiant as rd\n"
            ">>> rd.Authenticate()\n"
        )
